@@ -22,6 +22,50 @@ __all__ = [
     'postgres_context',
 ]
 
+#=========================================================
+# master containing all identifiable hashes
+#=========================================================
+def _load_master_config():
+    from passlib.registry import list_crypt_handlers
+
+    # get master list
+    schemes = list_crypt_handlers()
+
+    # exclude the ones we know have ambiguous or greedy identify() methods.
+    excluded = [
+        # frequently confused for eachother
+        'bigcrypt',
+        'crypt16',
+
+        # no good identifiers
+        'cisco_pix',
+        'cisco_type7',
+        'htdigest',
+        'mysql323',
+        'oracle10',
+
+        # all have same size
+        'lmhash',
+        'msdcc',
+        'msdcc2',
+        'nthash',
+
+        # plaintext handlers
+        'plaintext',
+        'ldap_plaintext',
+
+        # disabled handlers
+        'django_disabled',
+        'unix_disabled',
+        'unix_fallback',
+    ]
+    for name in excluded:
+        schemes.remove(name)
+
+    # return config
+    return dict(schemes=schemes, default="sha256_crypt")
+master_context = LazyCryptContext(onload=_load_master_config)
+
 #=========================================================
 #for quickly bootstrapping new custom applications
 #=========================================================
@@ -30,30 +74,41 @@ custom_app_context = LazyCryptContext(
     schemes=["sha512_crypt", "sha256_crypt"],
 
     #set some useful global options
-    all__vary_rounds = "10%",
     default="sha256_crypt" if sys_bits < 64 else "sha512_crypt",
+    all__vary_rounds = 0.1,
 
     #set a good starting point for rounds selection
-    sha512_crypt__default_rounds = 40000,
-    sha256_crypt__default_rounds = 40000,
+    sha512_crypt__min_rounds = 60000,
+    sha256_crypt__min_rounds = 80000,
 
     #if the admin user category is selected, make a much stronger hash,
-    admin__sha512_crypt__default_rounds = 80000,
-    admin__sha256_crypt__default_rounds = 80000,
+    admin__sha512_crypt__min_rounds = 120000,
+    admin__sha256_crypt__min_rounds = 160000,
     )
 
 #=========================================================
 #django
 #=========================================================
-django_context = LazyCryptContext(
-    schemes=[
+_django10_schemes = [
         "django_salted_sha1", "django_salted_md5", "django_des_crypt",
         "hex_md5", "django_disabled",
-    ],
+]
+
+django10_context = LazyCryptContext(
+    schemes=_django10_schemes,
     default="django_salted_sha1",
     deprecated=["hex_md5"],
 )
 
+django14_context = LazyCryptContext(
+    schemes=["django_pbkdf2_sha256", "django_pbkdf2_sha1", "django_bcrypt"] \
+            + _django10_schemes,
+    deprecated=_django10_schemes,
+)
+
+# this will always point to latest version
+django_context = django14_context
+
 #=========================================================
 #ldap
 #=========================================================
@@ -97,16 +152,13 @@ postgres_context = LazyCryptContext(["postgres_md5"])
 #phpass & variants
 #=========================================================
 def _create_phpass_policy(**kwds):
-    "helper to make bcrypt default ONLY if it's available"
-    from passlib.context import default_policy
-    if hash.bcrypt.has_backend():
-        kwds['default'] = 'bcrypt'
-    return default_policy.replace(**kwds)
+    "helper to choose default alg based on bcrypt availability"
+    kwds['default'] = 'bcrypt' if hash.bcrypt.has_backend() else 'phpass'
+    return kwds
 
 phpass_context = LazyCryptContext(
     schemes=["bcrypt", "phpass", "bsdi_crypt"],
-    default="phpass", #NOTE: <-- overridden by create_policy
-    create_policy=_create_phpass_policy,
+    onload=_create_phpass_policy,
     )
 
 phpbb3_context = LazyCryptContext(["phpass"], phpass__ident="H")
