@@ -10,7 +10,9 @@ from warnings import warn
 #site
 #libs
 #pkg
-from passlib.utils import handlers as uh, to_unicode, to_hash_str, bytes, b
+from passlib.utils import to_bytes
+from passlib.utils.compat import b, bytes, str_to_uascii, unicode, u
+import passlib.utils.handlers as uh
 #local
 __all__ = [
     "postgres_md5",
@@ -19,49 +21,33 @@ __all__ = [
 #=========================================================
 #handler
 #=========================================================
-class postgres_md5(uh.StaticHandler):
+class postgres_md5(uh.HasUserContext, uh.StaticHandler):
     """This class implements the Postgres MD5 Password hash, and follows the :ref:`password-hash-api`.
 
-    It has no salt and a single fixed round.
+    It does a single round of hashing, and relies on the username as the salt.
 
-    The :meth:`encrypt()` and :meth:`genconfig` methods accept no optional keywords.
-
-    The :meth:`encrypt()`, :meth:`genhash()`, and :meth:`verify()` methods all require the
+    The :meth:`~passlib.ifc.PasswordHash.encrypt`, :meth:`~passlib.ifc.PasswordHash.genhash`, and :meth:`~passlib.ifc.PasswordHash.verify` methods all require the
     following additional contextual keywords:
 
-    :param user: string containing name of postgres user account this password is associated with.
+    :type user: str
+    :param user: name of postgres user account this password is associated with.
     """
     #=========================================================
-    #algorithm information
+    # algorithm information
     #=========================================================
     name = "postgres_md5"
-    setting_kwds = ()
-    context_kwds = ("user",)
+    _hash_prefix = u("md5")
+    checksum_chars = uh.HEX_CHARS
+    checksum_size = 32
 
     #=========================================================
-    #formatting
+    # primary interface
     #=========================================================
-    _pat = re.compile(ur"^md5[0-9a-f]{32}$")
-
-    @classmethod
-    def identify(cls, hash):
-        return uh.identify_regexp(hash, cls._pat)
-
-    #=========================================================
-    #primary interface
-    #=========================================================
-    @classmethod
-    def genhash(cls, secret, config, user):
-        if config is not None and not cls.identify(config):
-            raise ValueError("not a postgres-md5 hash")
-        if not user:
-            raise ValueError("user keyword must be specified for this algorithm")
+    def _calc_checksum(self, secret):
         if isinstance(secret, unicode):
             secret = secret.encode("utf-8")
-        if isinstance(user, unicode):
-            user = user.encode("utf-8")
-        hash = u"md5" + to_unicode(md5(secret + user).hexdigest())
-        return to_hash_str(hash)
+        user = to_bytes(self.user, "utf-8", param="user")
+        return str_to_uascii(md5(secret + user).hexdigest())
 
     #=========================================================
     #eoc
